@@ -1,8 +1,10 @@
-#!/usr/bin/env python
+#!/usr/local/bin/python2.7
 ###############################################################################
+#  $Id: gdal_retile.py 21279 2010-12-16 21:25:08Z rouault $
 #
 # Purpose:  Module for retiling (merging) tiles and building tiled pyramids
 # Author:   Christian Meuller, christian.mueller@nvoe.at
+# UseDirForEachRow support by Chris Giesey & Elijah Robison
 #
 ###############################################################################
 # Copyright (c) 2007, Christian Mueller
@@ -78,11 +80,11 @@ class DataSetCache:
         self.dict={}
     def get(self,name ):
 
-        if self.dict.has_key(name):
+        if name in self.dict:
             return self.dict[name]
         result = gdal.Open(name)
         if result is None:
-            print "Error openenig:",NameError
+            print("Error openenig:%s" % NameError)
             sys.exit(1)
         if len(self.queue)==self.cacheSize:
             toRemove = self.queue.pop(0)
@@ -91,7 +93,7 @@ class DataSetCache:
         self.dict[name]=result
         return result
     def __del__(self):
-        for name, dataset in self.dict.iteritems():
+        for name, dataset in self.dict.items():
             del dataset
         del self.queue
         del self.dict
@@ -103,10 +105,10 @@ class tile_info:
     def __init__(self,xsize,ysize,tileWidth,tileHeight):
         self.tileWidth=tileWidth
         self.tileHeight=tileHeight
-        self.countTilesX= xsize / tileWidth
-        self.countTilesY= ysize / tileHeight
-        self.lastTileWidth = xsize - self.countTilesX *  tileWidth
-        self.lastTileHeight = ysize - self.countTilesY *  tileHeight
+        self.countTilesX= int(xsize / tileWidth)
+        self.countTilesY= int(ysize / tileHeight)
+        self.lastTileWidth = int(xsize - self.countTilesX *  tileWidth)
+        self.lastTileHeight = int(ysize - self.countTilesY *  tileHeight)
 
         if (self.lastTileWidth > 0 ):
             self.countTilesX=self.countTilesX+1
@@ -121,12 +123,12 @@ class tile_info:
 
 
     def report( self ):
-        print 'tileWidth       %d' % self.tileWidth
-        print 'tileHeight      %d' % self.tileHeight
-        print 'countTilesX:    %d' % self.countTilesX
-        print 'countTilesY:    %d' % self.countTilesY
-        print 'lastTileWidth:  %d' % self.lastTileWidth
-        print 'lastTileHeight: %d' % self.lastTileHeight
+        print('tileWidth       %d' % self.tileWidth)
+        print('tileHeight      %d' % self.tileHeight)
+        print('countTilesX:    %d' % self.countTilesX)
+        print('countTilesY:    %d' % self.countTilesY)
+        print('lastTileWidth:  %d' % self.lastTileWidth)
+        print('lastTileHeight: %d' % self.lastTileHeight)
 
 
 
@@ -164,6 +166,9 @@ class mosaic_info:
            self.ct = ct.Clone()
         else:
            self.ct = None
+        self.ci = [0] * self.bands
+        for iband in range(self.bands):
+            self.ci[iband] = fhInputTile.GetRasterBand(iband + 1).GetRasterColorInterpretation()
 
         extent = self.ogrTileIndexDS.GetLayer().GetExtent()
         self.ulx = extent[0];
@@ -247,6 +252,7 @@ class mosaic_info:
                 t_band = resultDS.GetRasterBand( bandNr )
                 if self.ct is not None:
                     t_band.SetRasterColorTable(self.ct)
+                t_band.SetRasterColorInterpretation(self.ci[bandNr-1])
                 data = s_band.ReadRaster( readOffsetX,readOffsetY,readX,readY, readX,readY, self.band_type )
                 t_band.WriteRaster(writeOffsetX,writeOffsetY,readX,readY,data )
 
@@ -258,19 +264,23 @@ class mosaic_info:
 
 
     def report( self ):
-        print 'Filename: '+ self.filename
-        print 'File Size: %dx%dx%d' \
-              % (self.xsize, self.ysize, self.bands)
-        print 'Pixel Size: %f x %f' \
-              % (self.scaleX,self.scaleY)
-        print 'UL:(%f,%f)   LR:(%f,%f)' \
-              % (self.ulx,self.uly,self.lrx,self.lry)
+        print('Filename: '+ self.filename)
+        print('File Size: %dx%dx%d' \
+              % (self.xsize, self.ysize, self.bands))
+        print('Pixel Size: %f x %f' \
+              % (self.scaleX,self.scaleY))
+        print('UL:(%f,%f)   LR:(%f,%f)' \
+              % (self.ulx,self.uly,self.lrx,self.lry))
 
 
 def getTileIndexFromFiles( inputTiles, driverTyp):
 
     if Verbose:
-        print "Building internal Index for %d tile(s) ..." % len(inputTiles),
+        from sys import version_info
+        if version_info >= (3,0,0):
+            exec('print("Building internal Index for %d tile(s) ..." % len(inputTiles), end=" ")')
+        else:
+            exec('print "Building internal Index for %d tile(s) ..." % len(inputTiles), ')
 
     ogrTileIndexDS = createTileIndex("TileIndex",TileIndexFieldName,None,driverTyp);
     for inputTile in inputTiles:
@@ -286,7 +296,7 @@ def getTileIndexFromFiles( inputTiles, driverTyp):
         del fhInputTile
 
     if Verbose:
-        print "finished"
+        print("finished")
     #ogrTileIndexDS.GetLayer().SyncToDisk()
     return ogrTileIndexDS
 
@@ -311,12 +321,13 @@ def tileImage(minfo, ti ):
 
     """
 
-
+    global LastRowIndx
+    LastRowIndx=-1
     OGRDS=createTileIndex("TileResult_0", TileIndexFieldName, Source_SRS,TileIndexDriverTyp)
 
 
-    yRange = range(1,ti.countTilesY+1)
-    xRange = range(1,ti.countTilesX+1)
+    yRange = list(range(1,ti.countTilesY+1))
+    xRange = list(range(1,ti.countTilesX+1))
 
     for yIndex in yRange:
         for xIndex in xRange:
@@ -331,16 +342,25 @@ def tileImage(minfo, ti ):
                 width=ti.lastTileWidth
             else:
                 width=ti.tileWidth
-            tilename=getTileName(minfo,ti, xIndex, yIndex)
+            if UseDirForEachRow :
+                tilename=getTileName(minfo,ti, xIndex, yIndex,0)
+            else:
+                tilename=getTileName(minfo,ti, xIndex, yIndex)
             createTile(minfo, offsetX, offsetY, width, height,tilename,OGRDS)
 
 
     if TileIndexName is not None:
-        shapeName=getTargetDir()+TileIndexName
+        if UseDirForEachRow and PyramidOnly == False:
+            shapeName=getTargetDir(0)+TileIndexName
+        else:
+            shapeName=getTargetDir()+TileIndexName
         copyTileIndexToDisk(OGRDS,shapeName)
 
     if CsvFileName is not None:
-        csvName=getTargetDir()+CsvFileName
+        if UseDirForEachRow and PyramidOnly == False:
+            csvName=getTargetDir(0)+CsvFileName
+        else:
+            csvName=getTargetDir()+CsvFileName
         copyTileIndexToCSV(OGRDS,csvName)
 
 
@@ -355,6 +375,9 @@ def copyTileIndexToDisk(OGRDS, fileName):
           break
       newFeature = feature.Clone()
       basename = os.path.basename(feature.GetField(0))
+      if UseDirForEachRow :
+          t = os.path.split(os.path.dirname(feature.GetField(0)))
+          basename = t[1]+"/"+basename
       newFeature.SetField(0,basename)
       SHAPEDS.GetLayer().CreateFeature(newFeature)
     closeTileIndex(SHAPEDS)
@@ -367,6 +390,9 @@ def copyTileIndexToCSV(OGRDS, fileName):
       if feature is None:
           break
       basename = os.path.basename(feature.GetField(0))
+      if UseDirForEachRow :
+          t = os.path.split(os.path.dirname(feature.GetField(0)))
+          basename = t[1]+"/"+basename
       csvfile.write(basename);
       geom = feature.GetGeometryRef()
       coords = geom.GetEnvelope();
@@ -417,20 +443,21 @@ def createPyramidTile(levelMosaicInfo, offsetX, offsetY, width, height,tileName,
         t_fh = MemDriver.Create( tileName, width, height, bands,bt,CreateOptions)
 
     if t_fh is None:
-        print 'Creation failed, terminating gdal_tile.'
+        print('Creation failed, terminating gdal_tile.')
         sys.exit( 1 )
 
 
     t_fh.SetGeoTransform( geotransform )
     t_fh.SetProjection( levelMosaicInfo.projection)
-    if levelMosaicInfo.ct is not None:
-        for band in range(1,bands+1):
-            t_band = t_fh.GetRasterBand( band )
+    for band in range(1,bands+1):
+        t_band = t_fh.GetRasterBand( band )
+        if levelMosaicInfo.ct is not None:
             t_band.SetRasterColorTable(levelMosaicInfo.ct)
+        t_band.SetRasterColorInterpretation(levelMosaicInfo.ci[band-1])
 
     res = gdal.ReprojectImage(s_fh,t_fh,None,None,ResamplingMethod)
     if  res!=0:
-        print "Reprojection failed for %s, error %d" % (tileName,res)
+        print("Reprojection failed for %s, error %d" % (tileName,res))
         sys.exit( 1 )
 
 
@@ -440,7 +467,7 @@ def createPyramidTile(levelMosaicInfo, offsetX, offsetY, width, height,tileName,
         tt_fh = Driver.CreateCopy( tileName, t_fh, 0, CreateOptions )
 
     if Verbose:
-        print tileName + " : " + str(offsetX)+"|"+str(offsetY)+"-->"+str(width)+"-"+str(height)
+        print(tileName + " : " + str(offsetX)+"|"+str(offsetY)+"-->"+str(width)+"-"+str(height))
 
 
 
@@ -487,7 +514,7 @@ def createTile( minfo, offsetX,offsetY,width,height, tilename,OGRDS):
         t_fh = MemDriver.Create( tilename, width, height, bands,bt,CreateOptions)
 
     if t_fh is None:
-        print 'Creation failed, terminating gdal_tile.'
+        print('Creation failed, terminating gdal_tile.')
         sys.exit( 1 )
 
     t_fh.SetGeoTransform( geotransform )
@@ -512,7 +539,7 @@ def createTile( minfo, offsetX,offsetY,width,height, tilename,OGRDS):
         tt_fh = Driver.CreateCopy( tilename, t_fh, 0, CreateOptions )
 
     if Verbose:
-        print tilename + " : " + str(offsetX)+"|"+str(offsetY)+"-->"+str(width)+"-"+str(height)
+        print(tilename + " : " + str(offsetX)+"|"+str(offsetY)+"-->"+str(width)+"-"+str(height))
 
 
 
@@ -520,7 +547,7 @@ def createTileIndex(dsName,fieldName,srs,driverName):
 
     OGRDriver = ogr.GetDriverByName(driverName);
     if OGRDriver is None:
-        print 'ESRI Shapefile driver not found'
+        print('ESRI Shapefile driver not found')
         sys.exit( 1 )
 
     OGRDataSource=OGRDriver.Open(dsName)
@@ -528,26 +555,26 @@ def createTileIndex(dsName,fieldName,srs,driverName):
         OGRDataSource.Destroy()
         OGRDriver.DeleteDataSource(dsName)
         if Verbose:
-            print 'truncating index '+ dsName
+            print('truncating index '+ dsName)
 
     OGRDataSource=OGRDriver.CreateDataSource(dsName)
     if OGRDataSource is None:
-        print 'Could not open datasource '+dsName
+        print('Could not open datasource '+dsName)
         sys.exit( 1 )
 
     OGRLayer = OGRDataSource.CreateLayer("index", srs, ogr.wkbPolygon)
     if OGRLayer is None:
-        print 'Could not create Layer'
+        print('Could not create Layer')
         sys.exit( 1 )
 
     OGRFieldDefn = ogr.FieldDefn(fieldName,ogr.OFTString)
     if OGRFieldDefn is None:
-        print 'Could not create FieldDefn for '+fieldName
+        print('Could not create FieldDefn for '+fieldName)
         sys.exit( 1 )
 
     OGRFieldDefn.SetWidth(256)
     if OGRLayer.CreateField(OGRFieldDefn) != 0:
-        print 'Could not create Field for '+fieldName
+        print('Could not create Field for '+fieldName)
         sys.exit( 1 )
 
     return OGRDataSource
@@ -557,7 +584,7 @@ def addFeature(OGRDataSource,location,xlist,ylist):
     OGRLayer=OGRDataSource.GetLayer();
     OGRFeature = ogr.Feature(OGRLayer.GetLayerDefn())
     if OGRFeature is None:
-        print 'Could not create Feature'
+        print('Could not create Feature')
         sys.exit( 1 )
 
     OGRFeature.SetField(TileIndexFieldName,location);
@@ -565,7 +592,7 @@ def addFeature(OGRDataSource,location,xlist,ylist):
             xlist[1],ylist[1],xlist[2],ylist[2],xlist[3],ylist[3],xlist[0],ylist[0])
     OGRGeometry=ogr.CreateGeometryFromWkt(wkt,OGRLayer.GetSpatialRef())
     if (OGRGeometry is None):
-        print 'Could not create Geometry'
+        print('Could not create Geometry')
         sys.exit( 1 )
 
     OGRFeature.SetGeometryDirectly(OGRGeometry)
@@ -579,16 +606,18 @@ def closeTileIndex(OGRDataSource):
 
 def buildPyramid(minfo,createdTileIndexDS,tileWidth, tileHeight):
 
+    global LastRowIndx
     inputDS=createdTileIndexDS
     for level in range(1,Levels+1):
+        LastRowIndx = -1
         levelMosaicInfo = mosaic_info(minfo.filename,inputDS)
         levelOutputTileInfo = tile_info(levelMosaicInfo.xsize/2,levelMosaicInfo.ysize/2,tileWidth,tileHeight)
         inputDS=buildPyramidLevel(levelMosaicInfo,levelOutputTileInfo,level)
 
 
 def buildPyramidLevel(levelMosaicInfo,levelOutputTileInfo, level):
-    yRange = range(1,levelOutputTileInfo.countTilesY+1)
-    xRange = range(1,levelOutputTileInfo.countTilesX+1)
+    yRange = list(range(1,levelOutputTileInfo.countTilesY+1))
+    xRange = list(range(1,levelOutputTileInfo.countTilesX+1))
 
     OGRDS=createTileIndex("TileResult_"+str(level), TileIndexFieldName, Source_SRS,TileIndexDriverTyp)
 
@@ -624,6 +653,8 @@ def getTileName(minfo,ti,xIndex,yIndex,level = -1):
     """
     creates the tile file name
     """
+    global LastRowIndx
+
     max = ti.countTilesX
     if (ti.countTilesY > max):
         max=ti.countTilesY
@@ -632,31 +663,42 @@ def getTileName(minfo,ti,xIndex,yIndex,level = -1):
     if parts[0][0]=="@" : #remove possible leading "@"
        parts = ( parts[0][1:len(parts[0])], parts[1])
 
-    if Extension is None:
-        format=getTargetDir(level)+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"+parts[1]
+    if UseDirForEachRow :
+        format=getTargetDir(level)+str(yIndex)+os.sep+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"
+        #See if there was a switch in the row, if so then create new dir for row.
+        if LastRowIndx < yIndex :
+            LastRowIndx = yIndex
+            if (os.path.exists(getTargetDir(level)+str(yIndex)) == False) :
+                os.mkdir(getTargetDir(level)+str(yIndex))
     else:
-        format=getTargetDir(level)+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"+"."+Extension
+        format=getTargetDir(level)+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"
+    #Check for the extension that should be used.
+    if Extension is None:
+        format=format+parts[1]
+    else:
+        format=format+"."+Extension
     return format % (yIndex,xIndex)
 
 def UsageFormat():
-    print 'Valid formats:'
+    print('Valid formats:')
     count = gdal.GetDriverCount()
     for index in range(count):
        driver= gdal.GetDriver(index)
-       print driver.ShortName
+       print(driver.ShortName)
 
 # =============================================================================
 def Usage():
-     print 'Usage: gdal_retile.py '
-     print '        [-v] [-co NAME=VALUE]* [-of out_format]'
-     print '        [-ps pixelWidth pixelHeight]'
-     print '        [-ot  {Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/'
-     print '               CInt16/CInt32/CFloat32/CFloat64}]'
-     print '        [ -tileIndex tileIndexName [-tileIndexField fieldName]]'
-     print '        [ -csv fileName [-csvDelim delimiter]]'
-     print '        [-s_srs srs_def]  [-pyramidOnly] -levels numberoflevels'
-     print '        [-r {near/bilinear/cubic/cubicspline}]'
-     print '        -targetDir TileDirectory input_files'
+     print('Usage: gdal_retile.py ')
+     print('        [-v] [-co NAME=VALUE]* [-of out_format]')
+     print('        [-ps pixelWidth pixelHeight]')
+     print('        [-ot  {Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/')
+     print('               CInt16/CInt32/CFloat32/CFloat64}]')
+     print('        [ -tileIndex tileIndexName [-tileIndexField fieldName]]')
+     print('        [ -csv fileName [-csvDelim delimiter]]')
+     print('        [-s_srs srs_def]  [-pyramidOnly] -levels numberoflevels')
+     print('        [-r {near/bilinear/cubic/cubicspline/lanczos}]')
+     print('        [-useDirForEachRow]')
+     print('        -targetDir TileDirectory input_files')
 
 # =============================================================================
 
@@ -666,7 +708,7 @@ def Usage():
 # Program mainline.
 #
 
-def main(args):
+def main(args = None):
 
     global Verbose
     global CreateOptions
@@ -689,9 +731,12 @@ def main(args):
     global ResamplingMethod
     global Levels
     global PyramidOnly
+    global UseDirForEachRow
 
     gdal.AllRegister()
 
+    if args is None:
+        args = sys.argv
     argv = gdal.GeneralCmdLineProcessor( args )
     if argv is None:
         return 1
@@ -708,7 +753,7 @@ def main(args):
             i+=1
             BandType = gdal.GetDataTypeByName( argv[i] )
             if BandType == gdal.GDT_Unknown:
-                print 'Unknown GDAL data type: ', argv[i]
+                print('Unknown GDAL data type: %s' % argv[i])
                 return 1
         elif arg == '-co':
             i+=1
@@ -723,7 +768,7 @@ def main(args):
             TargetDir=argv[i]
 
             if os.path.exists(TargetDir)==False:
-                print "TargetDir " + TargetDir + " does not exist"
+                print("TargetDir " + TargetDir + " does not exist")
                 return 1
             if TargetDir[len(TargetDir)-1:] != os.sep:
                 TargetDir =  TargetDir+os.sep
@@ -745,20 +790,22 @@ def main(args):
                  ResamplingMethod=GRA_Cubic
             elif ResamplingMethodString=="cubicspline":
                  ResamplingMethod=GRA_CubicSpline
+            elif ResamplingMethodString=="lanczos":
+                ResamplingMethod=GRA_Lanczos
             else:
-                print "Unknown resampling method:" ,ResamplingMethodString
+                print("Unknown resampling method: %s" % ResamplingMethodString)
                 return 1
         elif arg == '-levels':
             i+=1
             Levels=int(argv[i])
             if Levels<1:
-                print "Invalid number of levels", Levels
+                print("Invalid number of levels : %d" % Levels)
                 return 1
         elif arg == '-s_srs':
             i+=1
             Source_SRS = osr.SpatialReference()
             if Source_SRS.SetFromUserInput( argv[i] ) != 0:
-                print 'invalid -s_srs: ' + argv[i];
+                print('invalid -s_srs: ' + argv[i]);
                 return 1;
 
         elif arg ==  "-pyramidOnly":
@@ -782,8 +829,10 @@ def main(args):
         elif arg == '-csvDelim':
             i+=1
             CsvDelimiter=argv[i]
+        elif arg == '-useDirForEachRow':
+            UseDirForEachRow=True
         elif arg[:1] == '-':
-            print 'Unrecognised command option: ', arg
+            print('Unrecognised command option: %s' % arg)
             Usage()
             return 1
 
@@ -792,35 +841,42 @@ def main(args):
         i+=1
 
     if len(Names) == 0:
-        print 'No input files selected.'
+        print('No input files selected.')
         Usage()
         return 1
 
     if (TileWidth==0 or TileHeight==0):
-        print "Invalid tile dimension %d,%d" % (TileWidth,TileHeight)
+        print("Invalid tile dimension %d,%d" % (TileWidth,TileHeight))
         return 1
 
     if (TargetDir is None):
-        print "Missing Directory for Tiles -targetDir"
+        print("Missing Directory for Tiles -targetDir")
         Usage()
         return 1
 
+    # create level 0 directory if needed
+    if(UseDirForEachRow and PyramidOnly==False) :
+        leveldir=TargetDir+str(0)+os.sep
+        if (os.path.exists(leveldir)==False):
+            os.mkdir(leveldir)
+
     if Levels > 0:    #prepare Dirs for pyramid
-        for levelIndx in range (1,Levels+1):
+        startIndx=1
+        for levelIndx in range (startIndx,Levels+1):
             leveldir=TargetDir+str(levelIndx)+os.sep
             if (os.path.exists(leveldir)):
                 continue
             os.mkdir(leveldir)
             if (os.path.exists(leveldir)==False):
-                print "Cannot create level dir:", leveldir
+                print("Cannot create level dir: %s" % leveldir)
                 return 1
             if Verbose :
-                print "Created level dir: ",leveldir
+                print("Created level dir: %s" % leveldir)
 
 
     Driver = gdal.GetDriverByName(Format)
     if Driver is None:
-        print 'Format driver %s not found, pick a supported driver.' % Format
+        print('Format driver %s not found, pick a supported driver.' % Format)
         UsageFormat()
         return 1
 
@@ -829,13 +885,13 @@ def main(args):
 
     DriverMD = Driver.GetMetadata()
     Extension=DriverMD.get(DMD_EXTENSION);
-    if not DriverMD.has_key('DCAP_CREATE'):
+    if 'DCAP_CREATE' not in DriverMD:
         MemDriver=gdal.GetDriverByName("MEM")
 
 
     tileIndexDS=getTileIndexFromFiles(Names,TileIndexDriverTyp)
     if tileIndexDS is None:
-        print "Error building tile index"
+        print("Error building tile index")
         return 1;
     minfo = mosaic_info(Names[0],tileIndexDS)
     ti=tile_info(minfo.xsize,minfo.ysize, TileWidth, TileHeight)
@@ -843,7 +899,7 @@ def main(args):
     if Source_SRS is None and len(minfo.projection) > 0 :
        Source_SRS = osr.SpatialReference()
        if Source_SRS.SetFromUserInput( minfo.projection ) != 0:
-           print 'invalid projection  ' + minfo.projection;
+           print('invalid projection  ' + minfo.projection);
            return 1
 
     if Verbose:
@@ -861,7 +917,7 @@ def main(args):
        buildPyramid(minfo,dsCreatedTileIndex,TileWidth, TileHeight)
 
     if Verbose:
-        print "FINISHED"
+        print("FINISHED")
     return 0
 
 def initGlobals():
@@ -886,6 +942,9 @@ def initGlobals():
     global ResamplingMethod
     global Levels
     global PyramidOnly
+    global LastRowIndx
+    global UseDirForEachRow
+
 
     Verbose=False
     CreateOptions = []
@@ -908,6 +967,9 @@ def initGlobals():
     ResamplingMethod=GRA_NearestNeighbour
     Levels=0
     PyramidOnly=False
+    LastRowIndx=-1
+    UseDirForEachRow=False
+
 
 
 #global vars
@@ -931,9 +993,9 @@ TargetDir=None
 ResamplingMethod=GRA_NearestNeighbour
 Levels=0
 PyramidOnly=False
-
+LastRowIndx=-1
+UseDirForEachRow=False
 
 
 if __name__ == '__main__':
     sys.exit(main(sys.argv))
-
